# API Reference (v0.3.1)

 - [Reducer]()
  - [`reducer(schemas)`]()
 - [Middleware]()
  - [`middleware(config)`]()
 - [Schema]()
  - [`defineSchema(name, config)`]()
  - [`hasMany(name)`]()
  - [`generateSchemas(schemas)`]()
 - [Selectors]()
  - [`getEntities(state, schema)`]()
  - [`getEntitiesBy(state, schema, by={})`]()
  - [`getEntity(state, schema, id)`]()
 - [Helpers]()
  - [`createEntities(schema, dataPath, action)`]()
  - [`updateEntities(schema, ids, dataPath, action)`]()
  - [`updateEntityId(scehma, oldId, newId, action)`]()
  - [`deleteEntities(schema, id, action)`]()
  
## Reducer

#### `reducer(schemas)`

> Creates the reducer that will manage the *entities* slice of the store. It should be mounted at `entities`.

 - **Parameters**
  - `schemas` *Object*: Object containing entity schemas. Usually the output of [`generateSchemas(schemas)`](). 
 - **Returns**
  - *Function*: The reducer that will manage the *entities* slice of the store.

```javascript
import { reducer as entities } from 'entman';
import { combineReducers } from 'redux';
import { schemas } from './schemas';

const topReducer = combineReducers({
  // ...other reducers of the application
  entities: entities(schemas),
});
```

## Middleware

#### `middleware(config)`

> Creates the entman middleware needed to process the actions generated by the [helpers]().

 - **Parameters**
  - `config` *Object*: An object containing configuration options to pass to the middleware. Currently, the supported options are: 
    - `enableBatching`: Enable or disable batching of multiple entman actions into a single action. **Defaults to true**.
 - **Returns**
  - *Function*: A Redux middleware.

```javascript
import { createStore, applyMiddleware } from 'redux';
import { middleware as entman } from 'entman';
import reducer from './reducer';

export default createStore({
  reducer,
  applyMiddleware(entman({ enableBatching: true }),
});
```

## Schema

#### `defineSchema(name, config)`

> Creates an schema definition with the given named to be used in `generateSchemas`.

 - **Parameters**
  - `name` *String*: A string indicating the name of the entity this schema defines.
  - `config` *Object*: An object with the information to define the schema. The attributes are:
    - `attributes`: An object containing information about the relations of this entity to other entities and computed properties to be added to the entity when it's retrieved from the store.
 - **Returns**
  - *Object*: Schema definition of the entity.

```javascript
import { defineSchema } from 'entman';

const userDefinition = defineSchema('User', {
  group: 'Group',  // User belongs to Group

  // Define computed properties as functions
  getGroupName() { 
    return this.group.name;
  },
);
```

##### Note

When retrieving users from the store, they will contain defined computed properties. Inside computed properties we can compute data based on the entity and its relations. We can know for sure that in every place we're going to use that entity, the computed properties will be there, saving a lot imports around the application with functions to compute the same data.

#### `hasMany(name)`

> Defines an array like relationship with the entity identified by `name`.

 - **Parameters**
  - `name` *String*: A string indicating the name of the related entity.
 - **Returns**
  - *Object*: A relationship definition.

```javascript
import { defineSchema, hasMany } from 'entman';

const groupDefinition = defineSchema('Group', {
  users: hasMany('User')  // Group has many users
});
```

#### `generateSchemas(schemas)`

> Generates entities schemas from the definitions. The generated result is ready to be passed to the entities reducer.

 - **Parameters**
  - `schemas` *Array*: An array containing schemas definitions.
 - **Returns**
  - *Object*: An object with the schemas of the entities.

```javascript
import { defineSchema, generateSchemas } from 'entman';

const userDefinition = defineSchema('User');

export default generateSchemas([ userDefinition ]);
```

## Selectors

#### `getEntities(state, schema)`

> Get all the entities defined by `schema` from the state. It takes care of populate all the entities relationships and adding the computed properties defined in the schema.

 - **Parameters**
  - `state` *Object*: The global state of the application or an slice that contains the key `entities` on it.
  - `schema` *Object*: The schema of the entities to retrieve.
 - **Returns**
  - *Object*: An array with all the entities of the specified schema.
  
```javascript
import { getEntities } from 'entman';
import schemas from './schemas';

function getGroups(state) {
  return getEntities(state, schemas.Group);
}

// -----

const groups = getGroups(state);
```
  
#### `getEntitiesBy(state, schema, by={})`

> Get all the entities defined by `schema` from the state that match certain conditions. The conditions are specified by the `by` parameter which is an object that takes attributes of the entities as keys and the values these have to have as values to match. It takes care of populate all the entities relationships and adding the computed properties defined in the schema.

 - **Parameters**
  - `state` *Object*: The global state of the application or an slice that contains the key `entities` on it.
  - `schema` *Object*: The schema of the entities to retrieve.
  - `by` *Object*: An object specifying attributes of the entities and which value do they have to have. All entities matching those values are retrieved.
 - **Returns**
  - *Object*: An array with all the entities of the specified schema that match the conditions specified.
  
```javascript
import { getEntitiesBy } from 'entman';
import schemas from './schemas';

function getGroupsBy(state, by) {
  return getEntities(state, schemas.Group, by);
}

// -----

const groups = getGroupsBy(state, { name: 'Test' });
```
  
#### `getEntity(state, schema, id)`

> Get a single entity defined by `schema` with the specified `id` from the state. It takes care of populate all the entity relationships and adding the computed properties defined in the schema.

 - **Parameters**
  - `state` *Object*: The global state of the application or an slice that contains the key `entities` on it.
  - `schema` *Object*: The schema of the entities to retrieve.
  - `id` *String|Number*: The id of the entity to retrieve.
 - **Returns**
  - *Object*: The entity with the specified id.
  
```javascript
import { getEntity } from 'entman';
import schemas from './schemas';

function getGroup(state, id) {
  return getEntity(state, schemas.Group, id);
}

// -----

const group = getGroup(state, 1);
```

## Helpers

#### `createEntities(schema, dataPath, action)`

> 

 - **Parameters**:
  - `schema` *Object*:
  - `dataPath` *String*:
  - `action` *Object:
 - **Returns**:
  - *Object*:
  
```javascript
```

#### `updateEntities(schema, ids, dataPath, action)`

>

 - **Parameters**
  - `schema` *Object*:
  - `ids` *Array|Number|String*:
  - `dataPath` *String*:
  - `action` *Object*:
 - **Returns**:
  - *Object*:

```javascript
```

#### `updateEntityId(schema, oldId, newId, action)`

>

 - **Parameters**
  - `schema` *Object*:
  - `oldId` *Number|String*:
  - `newId` *Number|String*:
  - `action` *Object*:
 - **Returns**
  - *Object*:
  
```javascript
```

#### `deleteEntities(schema, id, action)`

> 

 - **Parameters**
  - `schema` *Object*:
  - `id` *Number|String*:
  - `action` *Object*:
  
```javascript
```
